/*---------------------------------------------------------------------------*/
/*                                                                           */
/*  Filename: PUTMSGQ.C                                                      */
/*                                                                           */
/*  Description:                                                             */
/*              This program is a companion program for MSGMON function. It  */
/*              reads an INI file for a Queue manager name, an IN Queue name */
/*              a Message Buffer filename, created by the Message Exit       */
/*              function, i.e. MSGMON that we created.  This program then    */
/*              connects to the Queue manager and opens the queue            */
/*              specified in the INI file, it reads the contents in          */
/*              the Message Buffer file and writes them to the message queue.*/
/*              This program is written to assist testing of programs        */
/*              working with MQSeries.                                       */
/*                                                                           */
/*              Note, an INI filename can be passed as an argument when      */
/*              invoking the program.  If no argument was specified,         */
/*              CHANNEL.INI under the \tmp directory will be used.           */
/*                                                                           */
/*  Statement: Licensed Materials - Property of IBM                          */
/*                                  Copyright IBM Corp. 1997                 */
/*                                  All rights reserved.                     */
/*                                                                           */
/*                                  U.S. Government Users Restricted Rights -*/
/*                                  use, duplication or disclosure restricted*/
/*                                  by GSA ADP Schedule Contract with IBM    */
/*                                  Corp.                                    */
/*                                                                           */
/*  Modification History:                                                    */
/*  Date        Who                Version   Description of change           */
/*  ----------  ---------------   ---------  -------------------------       */
/*  03/20/1997  Chak Pang         1.0        changed "[" to "??(" and "]"    */
/*                                           to "??)".  Also added comments  */
/*                                           suggested by Jeff Hooper and    */
/*                                           Harry Halliwell from Hursley,   */
/*                                           U.K.                            */
/*                                                                           */
/*  04/03/1997  Chak Pag          1.0        Tidied up comments and changed  */
/*                                           "[" to ""??(" and "]" to "??)". */
/*                                                                           */
/*  04/28/1997  Chak Pang         1.1        Changed INI_FILENAME to fn when */
/*                                           encountered an error opening    */
/*                                           the INI file.                   */
/*                                           Assigned pMQXQH and pMQMD ptr   */
/*                                           after reading the buffer file.  */
/*                                                                           */
/*  Authors:            Tom McCann                                           */
/*                      IBM Global Services                                  */
/*                      Cincinnati, Ohio, USA                                */
/*                      Phone: 513-762-2060                                  */
/*                                                                           */
/*                      Chak Pang                                            */
/*                      IBM Global Services                                  */
/*                      Columbus, Ohio, USA                                  */
/*                      Phone: 614-225-3663                                  */
/*                                                                           */
/*  Dependencies:       channel.h                                            */
/*                                                                           */
/*  Restrictions:       The INI filename must be less than 128 characters    */
/*                      The maximum size of a message is 10000 characters.   */
/*                                                                           */
/*  Disclaimer: this program has been tested under AIX and NT to the         */
/*              authors' satisfaction.  You may use this program at your own */
/*              risk.  IBM and the authors are not responsible for any       */
/*              unexpected results generated by this program.                */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#include <string.h>
#include <stdlib.h>
#include <cmqc.h>
#include <cmqxc.h>
#include <stdio.h>
#include <errno.h>

#include "channel.h"

long msgsize;
char *bufr;

   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   buflen;                 /* Length of message             */
   MQLONG   CReason;                /* reason code for MQCONN        */
   char     QMName??(50??);         /* queue manager name            */
   MQXQH    *pMQXQH;                /* message exit header           */
   MQMD     *pMQMD;                 /* message descriptor            */

   PMQXQH pAgentBuf;
   PMQMD pMsgDesc;  /*  Pointer to message descriptor */


char MsgBufFile??(129??);
char InQ??(129??);
char OutQ??(129??);
long offset;
FILE *msg_fp;

char buf??(10000??);  /* buffer to receive messages */
MQXQH agentBuf;
char ini_filename??(129??);

/*** Function prototpe for read subroutine ***/
/*   First argument is length of message,    */
/*   Second is returned buffer               */
/*   Returns int 0 if record read, 1 at EOF  */

int reader(long *pmsg, char *bfr);
void getINI( void );

/*-----------------------------------------------
   main
-------------------------------------------------*/
main( int argc, char ** argv )
{
   long count = 0L;

   memset( ini_filename, 0, sizeof( ini_filename ) );
   if ( argc > 1 )              /* if INI filename specified, copy it */
	strcpy( ini_filename, argv??(1??) );  

   getINI();

   pMQXQH = buf;        /* initialize pointers */
   pMQMD = &pMQXQH->MsgDesc;

printf("\nConnecting to Queue Mgr: ??(%s??)", QMName );

   MQCONN(QMName,                  /* queue manager                  */
	  &Hcon,                   /* connection handle              */
	  &CompCode,               /* completion code                */
	  &CReason);               /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     exit( (int)CReason );
   }

printf("\nConnected to %s", QMName);    

   /******************************************************************/
   /*                                                                */
   /*   Open the target message queue for output                     */
   /*                                                                */
   /******************************************************************/

   strcpy(od.ObjectName, InQ);
   O_options = MQOO_OUTPUT           /* open queue for output        */
	   + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping      */
	   
   MQOPEN(Hcon,                      /* connection handle            */
	  &od,                       /* object descriptor for queue  */
	  O_options,                 /* open options                 */
	  &Hobj,                     /* object handle                */
	  &OpenCode,                 /* MQOPEN completion code       */
	  &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("\nMQOPEN ended with reason code %ld\n", Reason);
     exit( 1 );
   }

   if (OpenCode == MQCC_FAILED)
      {
      printf("\nUnable to open queue for output\n");
      exit( 2 );
      }
   
   if ( (msg_fp = fopen( MsgBufFile, "rb" )) == NULL )
      {
      printf("\nError opening ??(%s??).  Error code: %d", MsgBufFile, errno );
      exit( 3 );
      }

   while (reader(&msgsize, buf) == 0) 
      {
      pMQXQH = buf;                   /* set pointers 4-28-97 */
      pMQMD = &pMQXQH->MsgDesc;       /* 4-28-97 */
      offset = sizeof(MQXQH);
      buflen = msgsize - offset;
      memcpy( &md, pMsgDesc, sizeof( md ) );   /* 4-28-97 */

      /****************************************************************/
      /*                                                              */
      /*   Put each buffer to the message queue                       */
      /*                                                              */
      /****************************************************************/
      if (buflen)
	 {
	 strcpy(md.ReplyToQ,OutQ); /* Dummy reply-to's */
	 strcpy(md.ReplyToQMgr, QMName);

         MQPUT(Hcon,              /* connection handle               */
	     Hobj,                /* object handle                   */
	     &md,                 /* message descriptor              */
	     &pmo,                /* default options (datagram)      */
	     buflen,              /* buffer length                   */
             &buf??(offset??),        /* message buffer                  */
	     &CompCode,           /* completion code                 */
	     &Reason);            /* reason code                     */

	 /* report reason, if any */
	 if (Reason != MQRC_NONE)
	    printf("MQPUT ended with reason code %ld\n", Reason);
	 else
	    printf("\nMQPUT %ld message in ??(%s??)", ++count, InQ );
	 } /* end if */
      } /* endwhile */
   fclose( msg_fp );
   return 0;
}

/*---------------------------------------------------------
   reader() - read information from Message Bufer file and
              writes it out to a queue
-----------------------------------------------------------*/
int reader(long *pmsg, char *bf)
{
   long msglength;
   char s1??(100??), s2??(100??);
  
   fread (&msglength, sizeof(msglength), 1, msg_fp);
   if (feof(msg_fp))
      return(1);
   
   *pmsg = msglength;
   fread (bf, (short)msglength, 1, msg_fp);

   pAgentBuf = bf;
   pMsgDesc = &pAgentBuf->MsgDesc;
   memset( s1, 0, sizeof( s1 ) );
   memset( s2, 0, sizeof( s2 ) );

   strncpy(s1, pAgentBuf->RemoteQName, sizeof( pAgentBuf->RemoteQName ));
   strncpy(s2, pAgentBuf->RemoteQMgrName, sizeof( pAgentBuf->RemoteQMgrName ));

printf("\n\nRemoteQName: ??(%s??)\nQueue Manager: ??(%s??)", s1, s2); 

   memset( s1, 0, sizeof( s1 ) );
   memset( s2, 0, sizeof( s2 ) );
   strncpy(s1, pMsgDesc->ReplyToQ, sizeof( pMsgDesc->ReplyToQ ));
   strncpy(s2, pMsgDesc->ReplyToQMgr, sizeof( pMsgDesc->ReplyToQMgr ));
printf("\nReplyToQ: ??(%s??)\nReplyToQMgr: ??(%s??)", s1, s2);
   
   memset( s1, 0, sizeof( s1 ) );
   memset( s2, 0, sizeof( s2 ) );
   memcpy(s1, pMsgDesc->MsgId, sizeof( pMsgDesc->MsgId ) );
   memcpy(s2, pMsgDesc->CorrelId, sizeof( pMsgDesc->CorrelId ) );
   printf("\nMsg Type: ??(%ld??)    MsgID: ??(%s??).   CorrelID: ??(%s??) ", 
	pMsgDesc->MsgType,s1, s2 );

   memset( s1, 0, sizeof( s1 ) );
   memset( s2, 0, sizeof( s2 ) );
   memcpy(s1, pMsgDesc->PutDate, sizeof( pMsgDesc->PutDate ) );
   memcpy(s2, pMsgDesc->PutTime, sizeof( pMsgDesc->PutTime )) ;
   
printf("\nPutDate: ??(%s??)   PutTime: ??(%s??)", s1, s2 );
  
   return(0);
}

/*---------------------------------------------------------
   getINI() - read information from INI file
-----------------------------------------------------------*/
void getINI( void )
{
   FILE *ini_fp;
   char *ptr, *name_ptr, field_val??(257??), Ini_s??(257??), fn??(129??);
   int i, len;
   
   memset( Ini_s, 0, sizeof( Ini_s ) );
   memset( field_val, 0, sizeof( field_val ) );
   memset( MsgBufFile, 0, sizeof ( MsgBufFile ) );
   memset( InQ, 0, sizeof( InQ ) );
   memset( OutQ, 0, sizeof( OutQ ) );
   memset( QMName, 0, sizeof( QMName ) );
   memset( fn, 0, sizeof( fn ) );
   if ( strlen( ini_filename ) )
	strcpy( fn, ini_filename );
   else
	strcpy( fn, INI_FILENAME );
   
   if ( (ini_fp = fopen( fn, "r" )) == NULL )
      {
      fprintf( stderr, "\nUnable to open %s.  Error #: %d", fn, errno );
      return;
      }
      
   /*-----------------------------*/
   /* read the INI file until eof */
   /*-----------------------------*/
   while ( (ptr = fgets( Ini_s, sizeof(Ini_s) - 1, ini_fp )) != NULL )
   {
      len = strlen( Ini_s );
      if ( Ini_s??(len -1??) == LINEFEED )
	Ini_s??(len-1??) = 0;                               /* null out CR */
      if ( (name_ptr = strchr( Ini_s, EQUAL )) == NULL )        /* no '=' found */
	 continue;
      ptr = name_ptr + 1;
      *name_ptr = 0;                                    /* set null for s??(??) */
      strcpy( field_val, ptr );

      /*---------------------------------------------------*/
      /* remove comments and unwanted characters from data */
      /*---------------------------------------------------*/
      while ( *ptr )
      {
	 if ( *ptr == ' ' )                     /* stop at first space */
	   {
	   *ptr = 0;
	   break;
	   }
	 ptr++;
      }
      
      if ( !strcmp( MESSAGE_BUFFER_FILE, Ini_s ) )  /* look for message buffer filename */
	 {
	 strcpy( MsgBufFile, field_val );
	 }
	  else
		if ( !strcmp( IN_QUE, Ini_s ) )                         /* look for IN Queue name */
			{
			strcpy( InQ, field_val );
			}
		else
			if ( !strcmp( OUT_QUE, Ini_s ) )                /* look for OUT Queue name */
			{
			strcpy( OutQ, field_val );
			}
			else
				if ( !strcmp( QUE_MGR, Ini_s ) )        /* look for Queue Mgr name */
				{
				strcpy( QMName, field_val );
				}
	memset( Ini_s, 0, sizeof( Ini_s ) );
	memset( field_val, 0, sizeof( field_val ) );
   } /* while ( fgets() ) */
   
   fclose( ini_fp );
   return;
}